@@ -22,43 +22,44 @@ def randomize(matrix):
 
 class Population:
     def __init__(self, size, hl_count, layer_node_counts):
-        self.creatures = []
+        self.birds = []
         for i in range(size):
-            self.creatures.append(Creature(hl_count, layer_node_counts))
+            self.birds.append(Bird(hl_count, layer_node_counts))
 
     def breed(self):
         odds = []
-        for i in self.creatures:
+        for i in self.birds:
             odds.append(i.fitness)
         odds = softmax(odds)
         
         next_gen = []
-        for i in range(len(self.creatures)):
-            p1 = np.random.choice(self.creatures, 1, p=odds)[0]
-            p2 = np.random.choice(self.creatures, 1, p=odds)[0]
+        next_gen = sorted(self.birds, key=lambda x: x.fitness)[-int(np.floor(len(self.birds) * .2)):]
+        while len(next_gen) < len(self.birds):
+            p1 = np.random.choice(self.birds, 1, p=odds)[0]
+            p2 = np.random.choice(self.birds, 1, p=odds)[0]
             mats = copy.copy(p1.matrices)
             for matrix in range(len(p1.matrices)):
                 for row in range(len(p1.matrices[matrix])):
                     for col in range(len(p1.matrices[matrix][0])):
                         if np.random.uniform() < .5:
                             mats[matrix][row][col] = p2.matrices[matrix][row][col]
-            next_gen.append(Creature(0, 0, mat=mats))
-        for i in range(len(self.creatures)):
-            del self.creatures[-1]
-        self.creatures = next_gen
+            next_gen.append(Bird(0, 0, mat=mats))
+        for i in range(len(self.birds)):
+            del self.birds[-1]
+        self.birds = next_gen
 
     def mutate(self, mutate_r8, radius_ratio):
-        for creature in self.creatures:
-            for matrix in range(len(creature.matrices)):
-                for row in range(len(creature.matrices[matrix])):
-                    for col in range(len(creature.matrices[matrix][0])):
+        for bird in self.birds:
+            for matrix in range(len(bird.matrices)):
+                for row in range(len(bird.matrices[matrix])):
+                    for col in range(len(bird.matrices[matrix][0])):
                         if np.random.uniform() < mutate_r8:
-                            creature.matrices[matrix][row][col] += np.random.normal(0, 1/np.sqrt(len(creature.matrices[matrix][0])) * radius_ratio, 1)[0]
+                            bird.matrices[matrix][row][col] += np.random.normal(0, 1/np.sqrt(len(bird.matrices[matrix][0])) * radius_ratio, 1)[0]
         
 
 
 
-class Creature:
+class Bird:
     def __init__(self, hl_count, layer_node_counts, mat = []):
         self.fitness = 0
         if mat:
@@ -75,3 +76,9 @@ class Creature:
         for matrix in self.matrices:
             hidden = sig(np.matmul(matrix, hidden))
         return hidden
+
+    def save(self, fname='./big-bird.json'):
+        record = []
+        for i in self.matrices:
+            record.append(i.tolist())
+        json.dump(record, codecs.open(fname, 'w', encoding='utf-8'), separators=(',', ':'), sort_keys=True, indent=4)
\ No newline at end of file
